@@ -1284,6 +1284,14 @@ static void drv_end(struct rfid_reader* reader)
   }
 }
 
+static int drv_fd(struct rfid_reader* reader)
+{
+  iso15693_t* iso15693 = (iso15693_t*)reader->opaque;
+  rfid_reader_t* driver = iso15693->reader;
+
+  return driver->fd_f(driver);
+}
+
 static char* drv_strerror(struct rfid_reader* reader)
 {
   iso15693_t* iso15693 = (iso15693_t*)reader->opaque;
@@ -1750,6 +1758,8 @@ static int drv_alloc(rfid_reader_t** readerp)
   reader->init_f = drv_init;
   reader->end_f = drv_end;
 
+  reader->fd_f = drv_fd;
+
   reader->strerror_f = drv_strerror;
   reader->error_f = drv_error;
   reader->version_f = drv_version;
