@@ -34,6 +34,10 @@
 #include <db.h>
 #include <rfid_reader.h>
 
+#if defined(__cplusplus)
+extern "C" {
+#endif
+
 /*
  * Implementation of ISO/IEC 15693-3:2001(E) & ISO/IEC 15693-2
  * for use as a host <-> reader communication protocol.
@@ -1238,7 +1242,7 @@ int iso15693_get_multiple_block_security_status(iso15693_t* iso15693, const u_in
 /*
  * Get access to iso15693 object.
  */
-#define iso15693_from_reader(reader) ((iso15693_t*)reader->private)
+#define iso15693_from_reader(reader) ((iso15693_t*)reader->opaque)
 
 /*
  * Return the size, in bytes, of the response to a request. The
@@ -1262,46 +1266,46 @@ int iso15693_crc16_check(u_int8_t* request, int request_length);
  * transponder t.
  */
 #define iso15693_transponder_info_flags_set(t,v) \
-        (((iso15693_transponder_t*)(t)->private)->info_flags = (v))
+        (((iso15693_transponder_t*)(t)->opaque)->info_flags = (v))
 /*
  * Copy the info flags (see ISO15693_INFO_FLAGS_* macros) from the transponder t
  * to v (u_int8_t).
  */
 #define iso15693_transponder_info_flags_get(t,v) \
-	((v) = ((iso15693_transponder_t*)(t)->private)->info_flags)
+	((v) = ((iso15693_transponder_t*)(t)->opaque)->info_flags)
 
 /*
  * Copy the UID from the v memory area (ISO15693_UID_SIZE bytes) to transponder t.
  */
 #define iso15693_transponder_uid_set(t,v) \
-	memcpy(((iso15693_transponder_t*)(t)->private)->uid, (v), ISO15693_UID_SIZE)
+	memcpy(((iso15693_transponder_t*)(t)->opaque)->uid, (v), ISO15693_UID_SIZE)
 /*
  * Copy the UID of transponder t to the v memory area (ISO15693_UID_SIZE bytes).
  */
 #define iso15693_transponder_uid_get(t,v) \
-	memcpy((v), ((iso15693_transponder_t*)(t)->private)->uid, ISO15693_UID_SIZE)
+	memcpy((v), ((iso15693_transponder_t*)(t)->opaque)->uid, ISO15693_UID_SIZE)
 
 /*
  * Copy the DSFID from the v memory area (ISO15693_DSFID_SIZE bytes) to transponder t.
  */
 #define iso15693_transponder_dsfid_set(t,v) \
-        (((iso15693_transponder_t*)(t)->private)->dsfid = (v))
+        (((iso15693_transponder_t*)(t)->opaque)->dsfid = (v))
 /*
  * Copy the DSFID of transponder t to the v memory area (ISO15693_DSFID_SIZE bytes).
  */
 #define iso15693_transponder_dsfid_get(t,v) \
-	((v) = ((iso15693_transponder_t*)(t)->private)->dsfid)
+	((v) = ((iso15693_transponder_t*)(t)->opaque)->dsfid)
 
 /*
  * Copy the AFI from the v memory area (ISO15693_AFI_SIZE bytes) to transponder t.
  */
 #define iso15693_transponder_afi_set(t,v) \
-        (((iso15693_transponder_t*)(t)->private)->afi = (v))
+        (((iso15693_transponder_t*)(t)->opaque)->afi = (v))
 /*
  * Copy the AFI of transponder t to the v memory area (ISO15693_AFI_SIZE bytes).
  */
 #define iso15693_transponder_afi_get(t,v) \
-	((v) = ((iso15693_transponder_t*)(t)->private)->afi)
+	((v) = ((iso15693_transponder_t*)(t)->opaque)->afi)
 
 /*
  * VICC memory size information bits:
@@ -1313,59 +1317,63 @@ int iso15693_crc16_check(u_int8_t* request, int request_length);
  * Copy the VICC from the v variable (u_int16_t) to the transponder t.
  */
 #define iso15693_transponder_vicc_memory_set(t,v) \
-        (((iso15693_transponder_t*)(t)->private)->vicc_memory = (v))
+        (((iso15693_transponder_t*)(t)->opaque)->vicc_memory = (v))
 /*
  * Copy the VICC of transponder t to the v variable (u_int16_t).
  */
 #define iso15693_transponder_vicc_memory_get(t,v) \
-	((v) = ((iso15693_transponder_t*)(t)->private)->vicc_memory)
+	((v) = ((iso15693_transponder_t*)(t)->opaque)->vicc_memory)
 
 /*
  * Set to 0 the blocks part of the VICC of the transponder t.
  */
 #define iso15693_transponder_vicc_blocks_clear(t) \
-        (((iso15693_transponder_t*)(t)->private)->vicc_memory &= 0xFF00)
+        (((iso15693_transponder_t*)(t)->opaque)->vicc_memory &= 0xFF00)
 /*
  * Copy the v variable (u_int8_t) to the transponder
  * blocks part of the VICC of the transponder t.
  */
 #define iso15693_transponder_vicc_blocks_set(t,v) \
-        (iso15693_transponder_vicc_blocks_clear(t), (((iso15693_transponder_t*)(t)->private)->vicc_memory |= ((v - 1) & 0xFF)))
+        (iso15693_transponder_vicc_blocks_clear(t), (((iso15693_transponder_t*)(t)->opaque)->vicc_memory |= ((v - 1) & 0xFF)))
 /*
  * Copy the blocks part of the VICC of the transponder to the v variable (u_int8_t).
  */
 #define iso15693_transponder_vicc_blocks_get(t,v) \
-	((v) = (((iso15693_transponder_t*)(t)->private)->vicc_memory & 0xFF) + 1)
+	((v) = (((iso15693_transponder_t*)(t)->opaque)->vicc_memory & 0xFF) + 1)
 
 /*
  * Set to 0 the bytes_per_block part of the VICC of the transponder t.
  */
 #define iso15693_transponder_vicc_bytes_per_block_clear(t) \
-        (((iso15693_transponder_t*)(t)->private)->vicc_memory &= 0xE0FF)
+        (((iso15693_transponder_t*)(t)->opaque)->vicc_memory &= 0xE0FF)
 /*
  * Copy the v variable (u_int8_t) to the transponder
  * bytes_per_block part of the VICC of the transponder t.
  */
 #define iso15693_transponder_vicc_bytes_per_block_set(t,v) \
-        (iso15693_transponder_vicc_bytes_per_block_clear(t), (((iso15693_transponder_t*)(t)->private)->vicc_memory |= (((v - 1) & 0x1F) << 8)))
+        (iso15693_transponder_vicc_bytes_per_block_clear(t), (((iso15693_transponder_t*)(t)->opaque)->vicc_memory |= (((v - 1) & 0x1F) << 8)))
 /*
  * Copy the bytes_per_block part of the VICC of the transponder
  * to the v variable (u_int8_t).
  */
 #define iso15693_transponder_vicc_bytes_per_block_get(t,v) \
-	((v) = ((((iso15693_transponder_t*)(t)->private)->vicc_memory >> 8) & 0x1F) + 1)
+	((v) = ((((iso15693_transponder_t*)(t)->opaque)->vicc_memory >> 8) & 0x1F) + 1)
 
 /*
  * Copy the IC REFERENCE from the v memory area
  * (ISO15693_IC_REFERENCE_SIZE bytes) to transponder t.
  */
 #define iso15693_transponder_ic_ref_set(t,v) \
-        (((iso15693_transponder_t*)(t)->private)->ic_ref = (v))
+        (((iso15693_transponder_t*)(t)->opaque)->ic_ref = (v))
 /*
  * Copy the IC REFERENCE of transponder t to the v memory area
  * (ISO15693_IC_REFERENCE_SIZE bytes).
  */
 #define iso15693_transponder_ic_ref_get(t,v) \
-	((v) = ((iso15693_transponder_t*)(t)->private)->ic_ref)
+	((v) = ((iso15693_transponder_t*)(t)->opaque)->ic_ref)
+
+#if defined(__cplusplus)
+}
+#endif
 
 #endif /* _ISO15693_H_ */
