@@ -1225,6 +1225,11 @@ static void drv_end(struct rfid_reader* reader)
   m2xxh_end(m2xxh);
 }
 
+static int drv_fd(struct rfid_reader* reader)
+{
+  return reader->io->fd;
+}
+
 static char* drv_strerror(struct rfid_reader* reader)
 {
   m2xxh_t* m2xxh = m2xxh_from_reader(reader);
@@ -1527,6 +1532,8 @@ static int drv_alloc(rfid_reader_t** readerp)
   reader->init_f = drv_init;
   reader->end_f = drv_end;
 
+  reader->fd_f = drv_fd;
+
   reader->strerror_f = drv_strerror;
   reader->error_f = drv_error;
   reader->version_f = drv_version;
