@@ -349,6 +349,13 @@ int rfid_transponder_id_set(rfid_t* rfid, rfid_transponder_t* transponder, char*
   return reader->transponder_id_set_f(reader, transponder, id);
 }
 
+int rfid_fd(rfid_t* rfid)
+{
+  rfid_reader_t* reader = rfid->reader;
+
+  return reader->fd_f(reader);
+}
+
 void rfid_end(rfid_t* rfid)
 {
   rfid->reader->end_f(rfid->reader);
@@ -410,16 +417,6 @@ static void transponders_sort(rfid_t* rfid, rfid_transponder_t** transponders, i
   rfid_qsort(transponders, transponders_length, sizeof(rfid_transponder_t*), (rfid_qsort_cmp)transponders_cmp, rfid);
 }
 
-typedef struct {
-  int events_mask;
-  rfid_transponder_callback_t* transponder_callback;
-  rfid_inventory_callback_t* inventory_callback;
-  void* data;
-  
-  rfid_transponders_t current;
-  rfid_transponders_t cache;
-} rfid_events_context_t;
-
 static int transponders_diff(rfid_t* rfid, rfid_events_context_t* context, int* changedp)
 {
   rfid_transponders_t* current = &context->current;
@@ -542,21 +539,12 @@ static int transponders_diff(rfid_t* rfid, rfid_events_context_t* context, int*
   return RFID_CALLBACK_OK;
 }
 
-static int event_loop(rfid_t* rfid, rfid_events_context_t* context)
+static int events_context_sanity_check(rfid_t* rfid)
 {
-  int retval = RFID_CALLBACK_OK;
-  /* 5 milliseconds */
-  struct timespec zzz = {
-    0,
-    5000000,
-  };
-  rfid_transponders_t* current = &context->current;
-  rfid_transponders_t* cache = &context->cache;
-  int events_mask = context->events_mask;
+  rfid_events_context_t* context = &rfid->events_context;
   rfid_inventory_callback_t* inventory_callback = context->inventory_callback;
   rfid_transponder_callback_t* transponder_callback = context->transponder_callback;
-  rfid_reader_t* reader = rfid->reader;
-  void* data = context->data;
+  int events_mask = context->events_mask;
 
   if((events_mask & (RFID_EVENT_NEW|RFID_EVENT_DELETE|RFID_EVENT_AGAIN)) &&
      !transponder_callback) {
@@ -572,81 +560,111 @@ static int event_loop(rfid_t* rfid, rfid_events_context_t* context)
     RFID_ERROR_STACK(rfid->verbose);
     return -1;
   }
-  
+
+  return 0;
+}
+
+#define RFID_CALLBACK_CONTINUE -2
+
+static int event_poll(rfid_t* rfid)
+{
+  rfid_events_context_t* context = &rfid->events_context;
+  int retval = RFID_CALLBACK_OK;
+  int changed = 0;
+  rfid_transponders_t* current = &context->current;
+  rfid_transponders_t* cache = &context->cache;
+  int events_mask = context->events_mask;
+  rfid_inventory_callback_t* inventory_callback = context->inventory_callback;
+  rfid_reader_t* reader = rfid->reader;
+  void* data = context->data;
+
   rfid->error = 0;
 
-  while(retval == RFID_CALLBACK_OK) {
-    int changed = 0;
-
-    if(block_signals(rfid) < 0) {
-      RFID_ERROR_STACK(rfid->verbose);
-      return -1;
-    }
-
-    if(reader->inventory_f(reader, current) < 0) {
-      int inventory_error = reader->error_f(reader);
-      if(release_signals(rfid) < 0) {
-	RFID_ERROR_STACK(rfid->verbose);
-	return -1;
-      }
-      /*
-       * Restart if a recoverable error occurs.
-       */
-      if(inventory_error == RFID_ERROR_TRY_AGAIN) {
-	if(rfid->verbose == 1) fprintf(stderr, "!");
-	continue;
-      }
-
-      RFID_ERROR_STACK(rfid->verbose);
-      return -1;
-    }
+  if(block_signals(rfid) < 0) {
+    RFID_ERROR_STACK(rfid->verbose);
+    return RFID_CALLBACK_ERROR;
+  }
 
+  if(reader->inventory_f(reader, current) < 0) {
+    int inventory_error = reader->error_f(reader);
     if(release_signals(rfid) < 0) {
       RFID_ERROR_STACK(rfid->verbose);
-      return -1;
+      return RFID_CALLBACK_ERROR;
     }
-
-    transponders_sort(rfid, current->list, current->length);
-
-    if((retval = transponders_diff(rfid, context, &changed)) < 0) {
-      /*
-       * Recoverable error (typically the state changed in the middle of
-       * the analysis of the current state).
-       */
-      if(rfid_error(rfid) == RFID_ERROR_TRY_AGAIN) {
-	if(rfid->verbose == 1) fprintf(stderr, "!");
-	rfid->error = 0;
-	retval = RFID_CALLBACK_OK;
-	continue;
-      }
-      /*
-       * Everything else is a fatal error, either from the rfid
-       * library or because a callback returned on error.
-       */
-      RFID_ERROR_STACK(rfid->verbose);
-      return -1;
-    }
-
-    if((retval == RFID_CALLBACK_OK) && changed && (events_mask & RFID_EVENT_INVENTORY)) {
-      if((retval = (*inventory_callback)(rfid, current->list, current->length, data)) < 0) {
-	RFID_ERROR_STACK(rfid->verbose);
-	return -1;
-      }
-    }
-    
-    if(rfid->verbose == 1) fprintf(stderr, "/");
-
     /*
-     * Swap cache & current transponders. 
+     * Restart if a recoverable error occurs.
      */
-    {
-      rfid_transponders_t tmp;
-
-      tmp = *cache;
-      *cache = *current;
-      *current = tmp;
+    if(inventory_error == RFID_ERROR_TRY_AGAIN) {
+      if(rfid->verbose == 1) fprintf(stderr, "!");
+      return RFID_CALLBACK_CONTINUE;
     }
 
+    RFID_ERROR_STACK(rfid->verbose);
+    return RFID_CALLBACK_ERROR;
+  }
+
+  if(release_signals(rfid) < 0) {
+    RFID_ERROR_STACK(rfid->verbose);
+    return RFID_CALLBACK_ERROR;
+  }
+
+  transponders_sort(rfid, current->list, current->length);
+
+  if((retval = transponders_diff(rfid, context, &changed)) < 0) {
+    /*
+     * Recoverable error (typically the state changed in the middle of
+     * the analysis of the current state).
+     */
+    if(rfid_error(rfid) == RFID_ERROR_TRY_AGAIN) {
+      if(rfid->verbose == 1) fprintf(stderr, "!");
+      rfid->error = 0;
+      return RFID_CALLBACK_CONTINUE;
+    }
+    /*
+     * Everything else is a fatal error, either from the rfid
+     * library or because a callback returned on error.
+     */
+    RFID_ERROR_STACK(rfid->verbose);
+    return RFID_CALLBACK_ERROR;
+  }
+
+  if((retval == RFID_CALLBACK_OK) && changed && (events_mask & RFID_EVENT_INVENTORY)) {
+    if((retval = (*inventory_callback)(rfid, current->list, current->length, data)) < 0) {
+      RFID_ERROR_STACK(rfid->verbose);
+      return RFID_CALLBACK_ERROR;
+    }
+  }
+    
+  if(rfid->verbose == 1) fprintf(stderr, "/");
+
+  /*
+   * Swap cache & current transponders. 
+   */
+  {
+    rfid_transponders_t tmp;
+
+    tmp = *cache;
+    *cache = *current;
+    *current = tmp;
+  }
+
+  return retval;
+}
+
+static int event_loop(rfid_t* rfid)
+{
+  int retval = RFID_CALLBACK_OK;
+  /* 5 milliseconds */
+  struct timespec zzz = {
+    0,
+    5000000,
+  };
+
+  rfid->error = 0;
+
+  while(retval == RFID_CALLBACK_OK || retval == RFID_CALLBACK_CONTINUE) {
+    retval = event_poll(rfid);
+
     /*
      * Short pause to avoid using all the CPU. More a safety measure
      * than really usefull since the speed of the serial line is more
@@ -658,22 +676,58 @@ static int event_loop(rfid_t* rfid, rfid_events_context_t* context)
   return 0;
 }
 
+int rfid_events_context_init(rfid_t* rfid, int events_mask, rfid_transponder_callback_t* transponder_callback, rfid_inventory_callback_t* inventory_callback, void* data)
+{
+  rfid_events_context_t* context = &rfid->events_context;
+
+  memset(context, '\0', sizeof(rfid_events_context_t));
+
+  context->events_mask = events_mask;
+  context->transponder_callback = transponder_callback;
+  context->inventory_callback = inventory_callback;
+  context->data = data;
+
+  return events_context_sanity_check(rfid);
+}
+
+int rfid_events_context_end(rfid_t* rfid)
+{
+  rfid_events_context_t* context = &rfid->events_context;
+
+  transponders_free(rfid, &context->current);
+  transponders_free(rfid, &context->cache);
+
+  return 0;
+}
+
+int rfid_event_poll(rfid_t* rfid)
+{
+  int retval = -1;
+
+  while((retval = event_poll(rfid)) == RFID_CALLBACK_CONTINUE)
+    ;
+
+  if(release_signals(rfid) < 0) {
+    RFID_ERROR_STACK(rfid->verbose);
+    return -1;
+  }
+
+  return retval;
+}
+
 int rfid_event_loop(rfid_t* rfid, int events_mask, rfid_transponder_callback_t* transponder_callback, rfid_inventory_callback_t* inventory_callback, void* data)
 {
   int retval = -1;
-  rfid_events_context_t context;
 
-  memset(&context, '\0', sizeof(rfid_events_context_t));
+  if(rfid_events_context_init(rfid, events_mask, transponder_callback, inventory_callback, data) < 0)
+    return -1;
+  
+  if(events_context_sanity_check(rfid) < 0)
+    return -1;
 
-  context.events_mask = events_mask;
-  context.transponder_callback = transponder_callback;
-  context.inventory_callback = inventory_callback;
-  context.data = data;
+  retval = event_loop(rfid);
 
-  retval = event_loop(rfid, &context);
-
-  transponders_free(rfid, &context.current);
-  transponders_free(rfid, &context.cache);
+  rfid_events_context_end(rfid);
 
   if(release_signals(rfid) < 0) {
     RFID_ERROR_STACK(rfid->verbose);
