@@ -44,12 +44,47 @@ extern "C" {
  */
 #define RFID_OPTION_FREE_IO		0x00000002
 
+struct rfid;
+/*
+ * Called when a new RFID transponder enters the range of the 
+ * RFID reader (event == RFID_EVENT_NEW), leaves the range of the
+ * RFID reader (event == RFID_EVENT_DELETE) or is still in the
+ * range of the RFID reader (event == RFID_EVENT_AGAIN).
+ * The transponder argument fully describes the RFID transponder,
+ * including meta information. The data argument is an opaque pointer
+ * identical to the data argument of the rfid_event_loop function.
+ */
+typedef int rfid_transponder_callback_t(struct rfid* rfid, int event, const rfid_transponder_t* transponder, void* data);
+
+/*
+ * Called at the end of each rfid_event_loop iteration. The
+ * transponders argument is an array of pointers to rfid_transponder_t
+ * objects that fully describe the RFID transponders including meta
+ * information. The data argument is an opaque pointer
+ * identical to the data argument of the rfid_event_loop function.
+ */
+typedef int rfid_inventory_callback_t(struct rfid* rfid, rfid_transponder_t** transponders, int transponders_length, void* data);
+
+/*
+ * Context of the event loop also used for polling.
+ */
+typedef struct {
+  int events_mask;
+  rfid_transponder_callback_t* transponder_callback;
+  rfid_inventory_callback_t* inventory_callback;
+  void* data;
+  
+  rfid_transponders_t current;
+  rfid_transponders_t cache;
+} rfid_events_context_t;
+
 /*
  * Handler holding context for a specific RFID device and used by
  * all rfid functions.
  */
-typedef struct {
+typedef struct rfid {
   rfid_reader_t* reader;	/* RFID reader handler. */
+  rfid_events_context_t events_context; /* State of the event loop. */
   int options;			/* Flags controling the behavior of
 				   rfid functions. */
   int error;			/* Errors set by rfid functions. */
@@ -101,6 +136,8 @@ int rfid_transponder_id_get(rfid_t* rfid, const rfid_transponder_t* transponder,
  */
 int rfid_transponder_id_set(rfid_t* rfid, rfid_transponder_t* transponder, char* id);
 
+int rfid_fd(rfid_t* rfid);
+
 /*
  * Callback functions for the rfid_event_loop function.
  */
@@ -137,35 +174,36 @@ int rfid_transponder_id_set(rfid_t* rfid, rfid_transponder_t* transponder, char*
 #define RFID_EVENT_NONE		0x00
 
 /*
- * Called when a new RFID transponder enters the range of the 
- * RFID reader (event == RFID_EVENT_NEW), leaves the range of the
- * RFID reader (event == RFID_EVENT_DELETE) or is still in the
- * range of the RFID reader (event == RFID_EVENT_AGAIN).
- * The transponder argument fully describes the RFID transponder,
- * including meta information. The data argument is an opaque pointer
- * identical to the data argument of the rfid_event_loop function.
+ * Prepare the context used by the rfid_event_poll and rfid_event_loop
+ * functions. The events_mask argument controls the calls to the
+ * transponder_callback and inventory_callback functions.  If the
+ * RFID_EVENT_NEW or RFID_EVENT_DELETE or RFID_EVENT_AGAIN bits are
+ * set, the transponder_callback function will be called. See the
+ * rfid_transponder_callback_t description for more information. If
+ * the RFID_EVENT_INVENTORY bit is set, the inventory_callback
+ * function will be called at the end of each iteration. See the
+ * rfid_inventory_callback_t description for more information. The
+ * data argument is an opaque pointer that is passed to the callback
+ * functions.
  */
-typedef int rfid_transponder_callback_t(rfid_t* rfid, int event, const rfid_transponder_t* transponder, void* data);
+int rfid_events_context_init(rfid_t* rfid, int events_mask, rfid_transponder_callback_t* transponder_callback, rfid_inventory_callback_t* inventory_callback, void* data);
 
 /*
- * Called at the end of each rfid_event_loop iteration. The
- * transponders argument is an array of pointers to rfid_transponder_t
- * objects that fully describe the RFID transponders including meta
- * information. The data argument is an opaque pointer
- * identical to the data argument of the rfid_event_loop function.
+ * Free the context allocated by rfid_events_context_init.
  */
-typedef int rfid_inventory_callback_t(rfid_t* rfid, rfid_transponder_t** transponders, int transponders_length, void* data);
+int rfid_events_context_end(rfid_t* rfid);
 
 /*
- * Infinite loop polling the RFID reader. The events_mask argument controls
- * the calls to the transponder_callback and inventory_callback functions.
- * If the RFID_EVENT_NEW or RFID_EVENT_DELETE or RFID_EVENT_AGAIN bits are
- * set, the transponder_callback function will be called. See the 
- * rfid_transponder_callback_t description for more information. If the
- * RFID_EVENT_INVENTORY bit is set, the inventory_callback function will be
- * called at the end of each iteration. See the rfid_inventory_callback_t
- * description for more information. The data argument is an opaque pointer
- * that is passed to the callback functions.
+ * Process a single event. The rfid_events_context_init function
+ * must be called prior to the first call to rfid_event_poll to
+ * specify events of interest and callbacks.
+ */
+int rfid_event_poll(rfid_t* rfid);
+
+/*
+ * Infinite loop polling the RFID reader. The arguments are
+ * fed to rfid_events_context_init and rfid_event_poll is
+ * called in an infinite loop.
  */
 int rfid_event_loop(rfid_t* rfid, int events_mask, rfid_transponder_callback_t* transponder_callback, rfid_inventory_callback_t* inventory_callback, void* data);
 
