@@ -119,12 +119,10 @@ static int run_daemon(commander_t* commander)
 static int add_command(commander_t* commander, int argc, char** argv)
 {
   rfid_t* rfid = commander->rfid;
-  rfid_transponder_t* transponder = 0;
+  rfid_transponder_t* transponder = rfid_transponder_alloc(rfid);
   int i;
   int retval = -1;
 
-  rfid->reader->transponder_alloc_f(rfid->reader, &transponder);
-
   for(i = 0; i < argc; i++) {
     int index = transponder->data_length;
     int length = strlen(argv[i]);
@@ -160,7 +158,7 @@ storage (maximum %d bytes).\n", transponder->data_length, RFID_TRANSPONDER_DATA_
  err:
 
   if(transponder)
-    rfid->reader->transponder_free_f(rfid->reader, transponder);
+    rfid_transponder_free(rfid, transponder);
   
   return retval;
 }
