@@ -35,6 +35,10 @@
 #include <sys/types.h>
 #include <termios.h>
 
+#if defined(__cplusplus)
+extern "C" {
+#endif
+
 #define RFID_IO_UNDEFINED	-1
 
 typedef struct {
@@ -62,4 +66,8 @@ int rfid_io_open(rfid_io_t* io);
 int rfid_io_read(rfid_io_t* io, u_int8_t* buffer, int buffer_size);
 int rfid_io_end(rfid_io_t* io);
 
+#if defined(__cplusplus)
+}
+#endif
+
 #endif /* _RFID_IO_H_ */
