@@ -123,6 +123,9 @@ typedef struct rfid_reader {
   /* Terminate connection with the reader device (before free_f). */
   void (*end_f)(struct rfid_reader* reader);
 
+  /* Return the file descriptor used to dialog with the RFID reader. */
+  int (*fd_f)(struct rfid_reader* reader);
+
   /* Return a malloc'ed string representation of the current error. */
   char* (*strerror_f)(struct rfid_reader* reader);
   /* Return the current error number. */
