@@ -907,6 +907,11 @@ static void drv_end(struct rfid_reader* reader)
   s6350_end(s6350);
 }
 
+static int drv_fd(struct rfid_reader* reader)
+{
+  return reader->io->fd;
+}
+
 static char* drv_strerror(struct rfid_reader* reader)
 {
   s6350_t* s6350 = s6350_from_reader(reader);
@@ -1613,6 +1618,8 @@ static int drv_alloc(rfid_reader_t** readerp)
   reader->init_f = drv_init;
   reader->end_f = drv_end;
 
+  reader->fd_f = drv_fd;
+
   reader->strerror_f = drv_strerror;
   reader->error_f = drv_error;
   reader->version_f = drv_version;
