@@ -35,6 +35,10 @@
 
 #include <rfid_reader.h>
 
+#if defined(__cplusplus)
+extern "C" {
+#endif
+
 /*
  * Handler holding context for a specific RFID device and used by all
  * s6350 function.
@@ -89,7 +93,7 @@ typedef struct {
 /*
  * Access s6350 internal structure.
  */
-#define s6350_from_reader(reader) ((s6350_t*)(reader)->private)
+#define s6350_from_reader(reader) ((s6350_t*)(reader)->opaque)
 
 /*
  * Returns the RFID transponder description in <transponder>, if one
@@ -145,4 +149,8 @@ int s6350_rf_carrier(s6350_t* s6350, u_int8_t on_off);
  */
 int s6350_baud_rate_configuration(s6350_t* s6350, u_int8_t bauds);
 
+#if defined(__cplusplus)
+}
+#endif
+
 #endif /* _S6350_H_ */
