@@ -117,11 +117,11 @@ FAILURE: Make sure a RFID reader is plugged in and turned on.\n\
 
 #define show_result(what) \
 	if(context->verbose) { \
-		fprintf(stderr, "%s : ", what); \
 		if(retval < 0) \
-			fprintf(stderr, " failed\n"); \
+			fprintf(stderr, "FAILED\t"); \
 		else \
-			fprintf(stderr, " OK\n"); \
+			fprintf(stderr, "OK\t"); \
+		fprintf(stderr, "%s\n", what); \
 	}
 
 static int test_inventory(context_t* context)
@@ -198,7 +198,9 @@ static int test_inventory_driver(context_t* context)
   }
 
   if(transponders.length < 1) {
-    fprintf(stderr, "Make sure there is at least one ISO transponder on the reader\n");
+    retval = -2;
+    if(context->verbose > 1)
+      fprintf(stderr, "test_inventory_driver: no transponder found\n");
     goto err;
   }
 
@@ -214,7 +216,7 @@ static int test_inventory_driver(context_t* context)
 
   /* Keep the first transponder as a reference for other tests. */
   {
-    iso15693_transponder_t* tmp = (iso15693_transponder_t*)transponders.list[0]->private;
+    iso15693_transponder_t* tmp = (iso15693_transponder_t*)transponders.list[0]->opaque;
     reader->transponder_copy_f(reader, context->transponder, transponders.list[0]);
     memcpy(context->uid, tmp->uid, ISO15693_UID_SIZE);
   }
@@ -230,7 +232,8 @@ static int test_inventory_driver(context_t* context)
 
   alarm(0);
 
-  show_result("inventory_driver");
+  if(retval != -2)
+    show_result("inventory_driver");
 
   return retval;
 }
@@ -590,7 +593,7 @@ static int test_lock_block(context_t* context)
   u_int8_t storage[ISO15693_BLOCK_SIZE_MAX];
 
   if(!context->destructive) {
-    if(context->verbose) fprintf(stderr, "lock_block : SKIP (destructive)\n");
+    if(context->verbose) fprintf(stderr, "SKIP\tlock_block (destructive)\n");
     return 0;
   }
 
@@ -736,7 +739,7 @@ static int test_lock_afi(context_t* context)
   int retval = -1;
 
   if(!context->destructive) {
-    if(context->verbose) fprintf(stderr, "lock_afi : SKIP (destructive)\n");
+    if(context->verbose) fprintf(stderr, "SKIP\tlock_afi (destructive)\n");
     return 0;
   }
 
@@ -860,7 +863,7 @@ static int test_lock_dsfid(context_t* context)
   int retval = -1;
 
   if(!context->destructive) {
-    if(context->verbose) fprintf(stderr, "lock_dsfid : SKIP (destructive)\n");
+    if(context->verbose) fprintf(stderr, "SKIP\tlock_dsfid (destructive)\n");
     return 0;
   }
 
@@ -1044,8 +1047,8 @@ static int test_stay_quiet(context_t* context)
 
 int main(int argc, char** argv)
 {
-  char* driver = 0;
-  char* device = 0;
+  char* driver = RFID_DRIVER_GUESS;
+  char* device = RFID_DEVICE_GUESS;
 
   if(argc > 1)
     driver = argv[1];
@@ -1055,17 +1058,22 @@ int main(int argc, char** argv)
   signal(SIGALRM, timeout_f);
 
   memset(&context, '\0', sizeof(context_t));
-  if(argc > 3)
+  if(getenv("DESTRUCTIVE"))
     context.destructive = 1;
   context.timeout = 5000;
-  context.rfid.verbose = context.verbose = 0;
+  context.rfid.verbose = 0;
+  context.verbose = 1;
 
   if(rfid_init(&context.rfid, driver, device) < 0) {
     report_error(&context, "rfid_init");
     return -1;
   }
   context.reader = context.rfid.reader;
-  context.iso15693 = (iso15693_t*)context.reader->private;
+
+  if(context.verbose)
+    fprintf(stderr, "Testing iso15693(3) with %s\n", context.reader->version_f(context.reader));
+
+  context.iso15693 = (iso15693_t*)context.reader->opaque;
 
   context.reader->transponder_alloc_f(context.reader, &context.transponder);
   memset(context.all_A, 'A', ISO15693_BLOCK_SIZE_MAX);
@@ -1074,7 +1082,18 @@ int main(int argc, char** argv)
   if(test_inventory(&context)) return -1;
   /* Registers the first transponder found for
      later tests (reference transponder). */
-  if(test_inventory_driver(&context)) return -1;
+  switch(test_inventory_driver(&context)) {
+  case 0:
+    break;
+  case -1:
+    return -1;
+    break;
+  case -2:
+    if(context.verbose)
+      fprintf(stderr, "SKIP\tcommands tests (no RFID transponder on the RFID reader).\n");
+    return 0;
+    break;
+  }
   /* Find out the memory layout of the reference
      transponder. */
   if(test_get_system_information(&context)) return -1;
