@@ -178,6 +178,7 @@ Wrote string %s to transponder:\n\
     free(transponder_description);
     rfid_transponder_free(&rfid, transponder);
   }
+  fprintf(stderr, "OK\trfid_write (non-addressed)\n");
   alarm(0);
 
   alarm(timeout);
@@ -193,6 +194,7 @@ Wrote string %s to transponder:\n\
     }
     rfid_transponder_free(&rfid, transponder);
   }
+  fprintf(stderr, "OK\trfid_write (addressed)\n");
   alarm(0);
   
   signal(SIGALRM, timeout_read);
@@ -201,12 +203,39 @@ Wrote string %s to transponder:\n\
   {
     int count = 3;
     if(rfid_event_loop(&rfid, RFID_EVENT_NEW|RFID_EVENT_AGAIN, transponder_callback, 0, &count) < 0) {
-      report_error(&rfid, "rfid_read");
+      report_error(&rfid, "rfid_event_loop");
       return -1;
     }
   }
+  fprintf(stderr, "OK\trfid_event_loop\n");
+  alarm(0);
+
+  alarm(timeout);
+  {
+    int count = 1;
+    if(rfid_events_context_init(&rfid, RFID_EVENT_NEW|RFID_EVENT_AGAIN, transponder_callback, 0, &count) < 0) {
+      report_error(&rfid, "rfid_events_context_init");
+      return -1;
+    }
+
+    if(rfid_event_poll(&rfid) < 0) {
+      report_error(&rfid, "rfid_event_poll");
+      return -1;
+    }
+
+    if(rfid_events_context_end(&rfid) < 0) {
+      report_error(&rfid, "rfid_events_context_end");
+      return -1;
+    }
+  }
+  fprintf(stderr, "OK\trfid_event_poll\n");
   alarm(0);
   
+  if(rfid_fd(&rfid) < 0) {
+    report_error(&rfid, "rfid_fd");
+    return -1;
+  }
+    
   rfid_end(&rfid);
   
   return 0;
